# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('appointments', '0002_initial'),
        ('base', '0005_initial'),
        ('careplans', '0004_initial'),
        ('devicerequests', '0004_initial'),
        ('medicationrequests', '0001_initial'),
        ('nutritionorders', '0001_initial'),
        ('patients', '0002_initial'),
        ('slots', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='appointment',
            name='slot',
            field=models.ManyToManyField(blank=True, related_name='appointment_slot', to='slots.slotreference'),
        ),
        migrations.AddField(
            model_name='appointment',
            name='specialty',
            field=models.ManyToManyField(blank=True, related_name='appointment_specialty', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='appointment',
            name='subject',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_subject', to='patients.patientgroupreference'),
        ),
        migrations.AddField(
            model_name='appointment',
            name='supporting_information',
            field=models.ManyToManyField(blank=True, related_name='appointment_supporting_information', to='base.reference'),
        ),
        migrations.AddField(
            model_name='appointment',
            name='virtual_service',
            field=models.ManyToManyField(blank=True, related_name='appointment_virtual_service_details', to='base.virtualservicedetails'),
        ),
        migrations.AddField(
            model_name='appointmentbasedonreference',
            name='care_plan',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_based_on_reference_care_plan', to='careplans.careplan'),
        ),
        migrations.AddField(
            model_name='appointmentbasedonreference',
            name='device_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_based_on_reference_device_request', to='devicerequests.devicerequest'),
        ),
        migrations.AddField(
            model_name='appointmentbasedonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_based_on_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='appointmentbasedonreference',
            name='medication_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_based_on_reference_medication_request', to='medicationrequests.medicationrequest'),
        ),
        migrations.AddField(
            model_name='appointmentbasedonreference',
            name='nutrition_order',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_based_on_reference_nutrition_order', to='nutritionorders.nutritionorder'),
        ),
    ]



"""EnrollmentResponses URL Configuration."""

from django.urls import path

from . import views

app_name = "enrollmentresponses"

urlpatterns = [
    path("enrollmentresponses/", views.EnrollmentResponseListView.as_view()),
    path("enrollmentresponses/<int:pk>/", views.EnrollmentResponseDetailView.as_view()),
]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('groups', '0003_initial'),
        ('plandefinitions', '0002_initial'),
        ('specimens', '0001_initial'),
        ('substances', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='plandefinitionactoroptiontypereferencereference',
            name='specimen',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='plan_definition_actor_option_type_reference_reference_specimen', to='specimens.specimen'),
        ),
        migrations.AddField(
            model_name='plandefinitionactoroptiontypereferencereference',
            name='substance',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='plan_definition_actor_option_type_reference_reference_substance', to='substances.substance'),
        ),
        migrations.AddField(
            model_name='plandefinitionactionparticipant',
            name='type_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_action_participant_type_reference', to='plandefinitions.plandefinitionactoroptiontypereferencereference'),
        ),
        migrations.AddField(
            model_name='plandefinitiongoal',
            name='addresses',
            field=models.ManyToManyField(related_name='plan_definition_goal_addresses', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='plandefinitiongoal',
            name='category',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_goal_category', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='plandefinitiongoal',
            name='description',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_goal_description', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='plandefinitiongoal',
            name='documentation',
            field=models.ManyToManyField(related_name='plan_definition_goal_documentation', to='base.relatedartifact'),
        ),
        migrations.AddField(
            model_name='plandefinitiongoal',
            name='priority',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_goal_priority', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='plandefinitiongoal',
            name='start',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_goal_start', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='plandefinition',
            name='goal',
            field=models.ManyToManyField(blank=True, related_name='plan_definition_goal', to='plandefinitions.plandefinitiongoal'),
        ),
        migrations.AddField(
            model_name='plandefinitiongoaltarget',
            name='detail_codeable_concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_goal_target_detail_codeable_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='plandefinitiongoaltarget',
            name='detail_quantity',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_goal_target_detail_quantity', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='plandefinitiongoaltarget',
            name='detail_range',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_goal_target_detail_range', to='base.range'),
        ),
        migrations.AddField(
            model_name='plandefinitiongoaltarget',
            name='detail_ratio',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_goal_target_detail_ratio', to='base.ratio'),
        ),
        migrations.AddField(
            model_name='plandefinitiongoaltarget',
            name='due',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_goal_target_due', to='base.period'),
        ),
        migrations.AddField(
            model_name='plandefinitiongoaltarget',
            name='measure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_goal_target_measure', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='plandefinitiongoal',
            name='target',
            field=models.ManyToManyField(related_name='plan_definition_goal_target', to='plandefinitions.plandefinitiongoaltarget'),
        ),
        migrations.AddField(
            model_name='plandefinitionsubjectreference',
            name='group',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='plan_definition_subject_reference_group', to='groups.group'),
        ),
        migrations.AddField(
            model_name='plandefinitionsubjectreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='plan_definition_subject_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='plandefinition',
            name='subject_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_subject_reference', to='plandefinitions.plandefinitionsubjectreference'),
        ),
        migrations.AddField(
            model_name='plandefintionactoroption',
            name='role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_actor_option_role', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='plandefintionactoroption',
            name='type_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_actor_option_type_reference', to='plandefinitions.plandefinitionactoroptiontypereferencereference'),
        ),
        migrations.AddField(
            model_name='plandefinitionactor',
            name='option',
            field=models.ManyToManyField(related_name='plan_definition_actor_option', to='plandefinitions.plandefintionactoroption'),
        ),
    ]



"""Devicerequests urls."""

from django.urls import path

from . import views

app_name = "devicerequests"


urlpatterns = [
    path("devicerequests/", views.DeviceRequestListView.as_view()),
    path("devicerequests/<int:pk>/", views.DeviceRequestDetailView.as_view()),
]



"""Practitioners URL Configuration."""

from django.urls import path

from . import views

app_name = "practitioners"

urlpatterns = [
    path("practitioners/", views.PractitionerListView.as_view()),
    path("practitioners/<int:pk>/", views.PractitionerDetailView.as_view()),
]



"""Clinical impressions models."""

from django.contrib.postgres.fields import ArrayField
from django.db import models

from dfhir.base.models import (
    Annotation,
    BaseReference,
    CodeableConcept,
    Identifier,
    Period,
    Reference,
    TimeStampedModel,
)

from . import choices


class ClinicalImpressionReference(BaseReference):
    """Clinical Impression reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="clinical_impression_reference_identifier",
        null=True,
    )
    clinical_impression = models.ForeignKey(
        "ClinicalImpression",
        on_delete=models.CASCADE,
        related_name="clinical_impression_reference_clinical_impression",
        null=True,
    )


class ClinicalImpressionFindingItemReference(BaseReference):
    """Clinical Impression Finding Item reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="clinical_impression_finding_item_reference_identifier",
        null=True,
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        on_delete=models.CASCADE,
        related_name="clinical_impression_finding_item_reference_condition",
        null=True,
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.CASCADE,
        related_name="clinical_impression_finding_item_reference_observation",
        null=True,
    )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.CASCADE,
        related_name="clinical_impression_finding_item_reference_document_reference",
        null=True,
    )


class ClinicalImpressionFindingItemCodeableReference(TimeStampedModel):
    """Clinical Impression Finding Item Codeable Reference model."""

    reference = models.ForeignKey(
        ClinicalImpressionFindingItemReference,
        on_delete=models.CASCADE,
        related_name="clinical_impression_finding_item_codeable_reference_reference",
        null=True,
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="clinical_impression_finding_item_codeable_reference_concept",
        null=True,
    )


class ClinicalImpressionFinding(TimeStampedModel):
    """Clinical Impression Finding model."""

    item = models.ForeignKey(
        ClinicalImpressionFindingItemCodeableReference,
        on_delete=models.CASCADE,
        related_name="clinical_impression_finding_item",
        null=True,
    )
    basis = models.TextField(null=True)


class ClinicalImpression(TimeStampedModel):
    """Clinical Impression model."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="clinical_impression_identifier",
        blank=True,
    )
    status = models.CharField(
        max_length=255, null=True, choices=choices.ClinicalImpressionStatus.choices
    )
    status_reason = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="clinical_impression_status_reason",
        null=True,
    )
    description = models.TextField(null=True)
    subject = models.ForeignKey(
        "patients.PatientGroupReference",
        on_delete=models.CASCADE,
        related_name="clinical_impression_subject",
        null=True,
    )
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.CASCADE,
        related_name="clinical_impression_encounter",
        null=True,
    )
    effective_date_time = models.DateTimeField(null=True)
    effective_period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="clinical_impression_effective_period",
        null=True,
    )
    date = models.DateTimeField(null=True)
    performer = models.ForeignKey(
        "practitioners.PractitionerPractitionerRoleReference",
        on_delete=models.CASCADE,
        related_name="clinical_impression_performer",
        null=True,
    )
    previous = models.ForeignKey(
        "ClinicalImpressionReference",
        on_delete=models.CASCADE,
        related_name="clinical_impression_previous",
        null=True,
    )
    problem = models.ManyToManyField(
        "conditions.ConditionAllergyIntoleranceReference",
        related_name="clinical_impression_problem",
        blank=True,
    )
    change_pattern = models.ForeignKey(
        CodeableConcept,
        related_name="clinical_impression_change_pattern",
        on_delete=models.CASCADE,
        null=True,
    )
    protocol = ArrayField(models.URLField(), null=True)
    summary = models.TextField(null=True)
    finding = models.ManyToManyField(
        ClinicalImpressionFinding,
        related_name="clinical_impression_finding",
        blank=True,
    )
    prognosis_codeable_concept = models.ManyToManyField(
        CodeableConcept,
        related_name="clinical_impression_prognosis_codeable_concept",
        blank=True,
    )
    # prognosis_reference = models.ManyToManyField(
    #     "riskassessments.RiskAssessment",
    #     related_name="clinical_impression_prognosis_reference",
    #     blank=True,
    # )
    supporting_info = models.ManyToManyField(
        Reference,
        related_name="clinical_impression_supporting_info",
        blank=True,
    )
    note = models.ManyToManyField(
        Annotation,
        related_name="clinical_impression_note",
        blank=True,
    )



"""Adverse events app configuration."""

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class AdverseeventsConfig(AppConfig):
    """Adverse events app config."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.adverseevents"
    verbose_name = _("Adverse Events")



"""immunization app models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    Reference,
    TimeStampedModel,
)
from dfhir.immunizations.choices import ImmunizationStatusChoices


class ImmunizationBasedOnReference(BaseReference):
    """Immunization based on reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_based_on_references",
    )
    care_plan = models.ForeignKey(
        "careplans.CarePlan",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_based_on_references",
    )
    medication_request = models.ForeignKey(
        "medicationrequests.MedicationRequest",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_based_on_references",
    )
    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_based_on_references",
    )
    immunization_recommendation = models.ForeignKey(
        "immunizationrecommendations.ImmunizationRecommendation",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_based_on_references",
    )


class ImmunizationInformationSourceRefrence(BaseReference):
    """immunization information source reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_information_source_references",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_information_source_references",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_information_source_references",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_information_source_references",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_information_source_references",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_information_source_references",
    )


class ImmunizationPerformerActorReference(BaseReference):
    """immunization performer actor reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_performer_actor_references",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_performer_actor_references",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_performer_actor_references",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_performer_actor_references",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_performer_actor_references",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_performer_actor_references",
    )


class ImmunizationPerformer(TimeStampedModel):
    """immunization performer model."""

    function = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_performers",
    )
    actor = models.ForeignKey(
        ImmunizationPerformerActorReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_performers",
    )


class ImmunizationReasonReference(BaseReference):
    """immunization reason reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_reason_references_identifier",
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_reason_references_condition",
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_reason_references_observation",
    )
    diagnostic_report = models.ForeignKey(
        "diagnosticreports.DiagnosticReport",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_reason_references_diagnostic_report",
    )


class ImmunizationReasonCodeableReference(TimeStampedModel):
    """immunization reason codeable reference model."""

    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_reason_codeable_references_identifier",
    )
    reference = models.ForeignKey(
        ImmunizationReasonReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_reason_codeable_references_reference",
    )


class ImmunizationProgramEligibility(TimeStampedModel):
    """immunization program eligibility model."""

    program = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_program_eligibility_program",
    )
    program_status = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_program_eligibility_program_status",
    )


class ImmunizationProtocolApplied(TimeStampedModel):
    """immunization protocol applied model."""

    series = models.CharField(max_length=255, null=True)
    authority = models.ForeignKey(
        "base.OrganizationReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_protocol_applied_authority",
    )
    target_disease = models.ManyToManyField(
        CodeableConcept,
        blank=True,
        related_name="immunization_protocol_applied_usage_disease",
    )
    dose_number = models.CharField(max_length=255, null=True)
    series_dose = models.CharField(max_length=255, null=True)


class ImmunizationReaction(TimeStampedModel):
    """immunization reaction model."""

    date = models.DateTimeField(null=True)
    manifestation = models.ForeignKey(
        "observations.ObservationCodeableReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_reaction_detail",
    )
    reported = models.BooleanField(default=False)


class Immunization(TimeStampedModel):
    """immunization model."""

    identifier = models.ManyToManyField(
        Identifier, blank=True, related_name="immunization_identifier"
    )
    based_on = models.ManyToManyField(
        ImmunizationBasedOnReference, blank=True, related_name="immunization_based_on"
    )
    status = models.CharField(
        max_length=255, null=True, choices=ImmunizationStatusChoices.choices
    )
    status_reason = models.ForeignKey(
        CodeableConcept,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="immunization_status_reason",
    )
    vaccine_code = models.ForeignKey(
        CodeableConcept,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="immunization_vaccine_code",
    )
    administered_product = models.ForeignKey(
        "medications.MedicationCodeableReference",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="immunization_administered_product",
    )
    manufacturer = models.ForeignKey(
        "organizations.OrganizationCodeableReference",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="immunization_manufacturer",
    )
    lot_number = models.CharField(max_length=255, null=True)
    expiration_date = models.DateField(null=True)
    patient = models.ForeignKey(
        "patients.PatientReference",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="immunization_patient",
    )
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="immunization_encounter",
    )
    supporting_infortmation = models.ManyToManyField(
        Reference, blank=True, related_name="immunization_supporting_infortmation"
    )
    occurrence_date_time = models.DateTimeField(null=True)
    occurrence_string = models.CharField(max_length=255, null=True)
    primary_source = models.BooleanField(default=False)
    information_source = models.ForeignKey(
        ImmunizationInformationSourceRefrence,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="immunization_information_source",
    )
    location = models.ForeignKey(
        "locations.LocationReference",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="immunization_location",
    )
    site = models.ForeignKey(
        CodeableConcept,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="immunization_site",
    )
    route = models.ForeignKey(
        CodeableConcept,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="immunization_route",
    )
    dose_quantity = models.ForeignKey(
        "base.SimpleQuantity",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="immunization_dose_quantity",
    )
    performer = models.ManyToManyField(
        ImmunizationPerformer, blank=True, related_name="immunization_performer"
    )
    note = models.ManyToManyField(
        "base.Annotation", blank=True, related_name="immunization_note"
    )
    reason = models.ManyToManyField(
        ImmunizationReasonCodeableReference,
        blank=True,
        related_name="immunization_reason",
    )
    is_subpotent = models.BooleanField(default=False)
    subpotent_reason = models.ManyToManyField(
        CodeableConcept, blank=True, related_name="immunization_subpotent_reason"
    )
    program_eligibility = models.ManyToManyField(
        ImmunizationProgramEligibility,
        blank=True,
        related_name="immunization_program_eligibility",
    )
    funding_source = models.ForeignKey(
        CodeableConcept,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="immunization_funding_source",
    )
    reaction = models.ManyToManyField(
        ImmunizationReaction, blank=True, related_name="immunization_reaction"
    )
    protocol_applied = models.ManyToManyField(
        ImmunizationProtocolApplied,
        blank=True,
        related_name="immunization_protocol_applied",
    )


class ImmunizationReference(BaseReference):
    """immunization reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_reference_identifier",
    )
    immunization = models.ForeignKey(
        Immunization,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_reference_immunization",
    )



"""imaging study serializers."""

from drf_writable_nested import WritableNestedModelSerializer
from rest_framework import serializers

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
)
from dfhir.bodystructures.serializers import BodyStructureCodeableReferenceSerializer
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.endpoints.serializers import EndpointReferenceSerializer
from dfhir.imagingstudies.models import (
    ImagingStudy,
    ImagingStudyBasedOnReference,
    ImagingStudyProcedureReference,
    ImagingStudyReasonReference,
    ImagingStudyReference,
    ImagingStudySeries,
    ImagingStudySeriesInstance,
    ImagingStudySeriesPerformer,
    ImagingStudySeriesPerformerActorReference,
    ImagingStudySubjectReference,
)
from dfhir.locations.serializers import LocationReferenceSerializer
from dfhir.practitioners.serializers import (
    PractitionerPractitionerRoleReferenceSerializer,
)
from dfhir.specimens.serializers import SpecimenReferenceSerializer


class ImagingStudySubjectReferenceSerializer(BaseReferenceModelSerializer):
    """ImagingStudySubjectReferenceSerializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta."""

        model = ImagingStudySubjectReference
        exclude = ["created_at", "updated_at"]


class ImagingStudyBasedOnReferenceSerializer(BaseReferenceModelSerializer):
    """Imaging Study Based On Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta."""

        model = ImagingStudyBasedOnReference
        exclude = ["created_at", "updated_at"]


class ImagingStudyProcedureReferenceSerializer(BaseReferenceModelSerializer):
    """ImagingStudyProcedureReferenceSerializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta."""

        model = ImagingStudyProcedureReference
        exclude = ["created_at", "updated_at"]


class ImagingStudyReasonReferenceSerializer(BaseReferenceModelSerializer):
    """imaging study reason reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta."""

        model = ImagingStudyReasonReference
        exclude = ["created_at", "updated_at"]


class ImagingStudySeriesPerformerActorReferenceSerializer(BaseReferenceModelSerializer):
    """ImagingStudySeriesPerformerActorReferenceSerializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta."""

        model = ImagingStudySeriesPerformerActorReference
        exclude = ["created_at", "updated_at"]


class ImagingStudySeriesPerformerSerializer(WritableNestedModelSerializer):
    """ImagingStudySeriesPerformerSerializer."""

    function = CodeableConceptSerializer(required=False)
    actor = ImagingStudySeriesPerformerActorReferenceSerializer(required=False)

    class Meta:
        """Meta."""

        model = ImagingStudySeriesPerformer
        exclude = ["created_at", "updated_at"]


class ImagingStudySeriesInstanceSerializer(serializers.ModelSerializer):
    """imaging study series instance serializer."""

    class Meta:
        """meta."""

        model = ImagingStudySeriesInstance
        exclude = ["created_at", "updated_at"]


class ImagingStudySeriesSerializer(WritableNestedModelSerializer):
    """imaging study series serializer."""

    modality = CodeableConceptSerializer(required=False)
    endpoint = EndpointReferenceSerializer(required=False, many=True)
    body_site = BodyStructureCodeableReferenceSerializer(required=False)
    laterality = CodeableConceptSerializer(required=False)
    specimen = SpecimenReferenceSerializer(required=False, many=True)
    performer = ImagingStudySeriesPerformerSerializer(required=False, many=True)
    instance = ImagingStudySeriesInstanceSerializer(required=False, many=True)

    class Meta:
        """meta."""

        model = ImagingStudySeries
        exclude = ["created_at", "updated_at"]


class ImagingStudySerializer(BaseWritableNestedModelSerializer):
    """ImagingStudySerializer."""

    identifier = IdentifierSerializer(required=False, many=True)
    modality = CodeableConceptSerializer(required=False, many=True)
    subject = ImagingStudySubjectReferenceSerializer(required=False)
    encounter = EncounterReferenceSerializer(required=False)
    based_on = ImagingStudyBasedOnReferenceSerializer(required=False, many=True)
    part_of = ImagingStudyBasedOnReferenceSerializer(required=False, many=True)
    referrer = PractitionerPractitionerRoleReferenceSerializer(required=False)
    endpoint = EndpointReferenceSerializer(required=False, many=True)
    procedure = ImagingStudyProcedureReferenceSerializer(required=False, many=True)
    location = LocationReferenceSerializer(required=False)
    reason = ImagingStudyReasonReferenceSerializer(required=False, many=True)
    note = AnnotationSerializer(required=False, many=True)
    series = ImagingStudySeriesSerializer(required=False, many=True)

    class Meta:
        """Meta."""

        model = ImagingStudy
        exclude = ["created_at", "updated_at"]


class ImagingStudyReferenceSerializer(BaseReferenceModelSerializer):
    """Imaging Selection Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = ImagingStudyReference
        exclude = ["created_at", "updated_at"]



"""Practitioner views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .filters import PractitionerFilter
from .models import (
    Practitioner,
)
from .serializers import (
    PractitionerSerializer,
)


class PractitionerListView(APIView):
    """Practitioner list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: PractitionerSerializer(many=True)})
    def get(self, request, pk=None):
        """Get practitioners."""
        queryset = Practitioner.objects.all()
        practitioners_filter = PractitionerFilter(request.GET, queryset=queryset)
        serializer = PractitionerSerializer(practitioners_filter.qs, many=True)
        return Response(serializer.data)

    @extend_schema(
        request=PractitionerSerializer, responses={200: PractitionerSerializer}
    )
    def post(self, request):
        """Create a practitioner."""
        request_data = request.data
        request_data.pop("user", None)

        serializer = PractitionerSerializer(data=request_data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)


class PractitionerDetailView(APIView):
    """Practitioner detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get practitioner object."""
        try:
            return Practitioner.objects.get(pk=pk)
        except Practitioner.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: PractitionerSerializer})
    def get(self, request, pk=None):
        """Get a practitioner."""
        queryset = self.get_object(pk)
        serializer = PractitionerSerializer(queryset)
        return Response(serializer.data)

    @extend_schema(responses={200: PractitionerSerializer})
    def patch(self, request, pk=None):
        """Update a practitioner."""
        queryset = self.get_object(pk)
        serializer = PractitionerSerializer(queryset, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)

    def delete(self, request, pk=None):
        """Delete a practitioner."""
        practitioner = self.get_object(pk)
        practitioner.user.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
import django.db.models.fields.related
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('encounters', '0002_initial'),
        ('medications', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='MedicationStatementAdherence',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationStatementInformationSourceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationStatementPartOfReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationStatementReasonReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationStatement',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('recorded', 'Recorded'), ('entered-in-error', 'Entered in error'), ('draft', 'Draft')], max_length=255, null=True)),
                ('effective_date_time', models.DateTimeField(null=True)),
                ('date_asserted', models.DateTimeField(null=True)),
                ('related_clinical_information', models.TextField(null=True)),
                ('category', models.ManyToManyField(blank=True, related_name='medication_statement_category', to='base.codeableconcept')),
                ('derived_from', models.ManyToManyField(blank=True, related_name='medication_statement_derived_from', to='base.reference')),
                ('effective_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_statement_effective_period', to='base.period')),
                ('effective_timing', models.ForeignKey(null=True, on_delete=django.db.models.fields.related.ForeignKey, related_name='medication_statement_effective_timing', to='base.timing')),
                ('encounter', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_statement_encounter', to='encounters.encounterreference')),
                ('identifier', models.ManyToManyField(blank=True, related_name='medication_statement_identifier', to='base.identifier')),
                ('medication', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_statement_medication', to='medications.medicationcodeablereference')),
                ('note', models.ManyToManyField(blank=True, related_name='medication_statement_note', to='base.annotation')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Accounts app configuration."""

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class AccountsConfig(AppConfig):
    """Accounts config."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.accounts"
    verbose_name = _("Accounts")



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0004_initial'),
        ('endpoints', '0001_initial'),
        ('healthcareservices', '0001_initial'),
        ('locations', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='PractitionerRoleCode',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('display', models.CharField(max_length=255)),
                ('definition', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PractitionerRoleReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PractitionerRole',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('active', models.BooleanField(default=True)),
                ('display', models.TextField(null=True)),
                ('availability', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='practitioner_role_availability', to='base.availability')),
                ('characteristic', models.ManyToManyField(blank=True, related_name='practitioner_role_characteristic', to='base.codeableconcept')),
                ('code', models.ManyToManyField(blank=True, related_name='practitioner_role_code', to='base.codeableconcept')),
                ('communication', models.ManyToManyField(blank=True, related_name='practitioner_role_communication', to='base.codeableconcept')),
                ('contact', models.ManyToManyField(blank=True, related_name='practitioner_role_contact', to='base.extendedcontactdetail')),
                ('endpoint', models.ManyToManyField(blank=True, related_name='practitioner_role_endpoint', to='endpoints.endpointreference')),
                ('healthcare_service', models.ManyToManyField(related_name='practitioner_role_healthcareservice', to='healthcareservices.healthcareservicereference')),
                ('identifier', models.ManyToManyField(blank=True, related_name='practitioner_role_identifier', to='base.identifier')),
                ('location', models.ManyToManyField(blank=True, related_name='practitioner_role_location', to='locations.locationreference')),
                ('network', models.ManyToManyField(blank=True, related_name='practitioner_role_network', to='base.organizationreference')),
                ('organization', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='practitioner_role_organization', to='base.organizationreference')),
                ('period', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='base.period')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""immunization app urls."""

from django.urls import path

from dfhir.immunizations import views

app_name = "immunizations"
urlpatterns = [
    path(
        "immunizations/",
        views.ImmunizationListView.as_view(),
        name="immunization-list-create",
    ),
    path(
        "immunizations/<int:pk>/",
        views.ImmunizationDetailView.as_view(),
        name="immunization-retrieve-update-destroy",
    ),
]



"""Biologivally Derived Products App Configuration."""

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class BiologicallyderivedproductsConfig(AppConfig):
    """Biologically Derived Products app configuration."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.biologicallyderivedproducts"
    verbose_name = _("Biologically Derived Products")



"""Enrollment Requests App Configuration."""

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class EnrollmentrequestsConfig(AppConfig):
    """Enrollment Requests App Configuration."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.enrollmentrequests"
    verbose_name = _("Enrollment Requests")



"""Care Team choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class CareTeamStatusChoices(models.TextChoices):
    """Care Team status choices."""

    PROPOSED = "proposed", _("Proposed")
    ACTIVE = "active", _("Active")
    SUSPENDED = "suspended", _("Suspended")
    INACTIVE = "inactive", _("Inactive")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")



"""Location filters."""

from django_filters import rest_framework as filters

from dfhir.locations.models import Location


class LocationFilter(filters.FilterSet):
    """Location filter."""

    name = filters.CharFilter(lookup_expr="icontains")
    location_status = filters.CharFilter(lookup_expr="icontains")
    organization = filters.CharFilter(field_name="organizations")
    service = filters.CharFilter(field_name="location_service_type")

    class Meta:
        """Meta class."""

        model = Location
        fields = ["name", "location_status"]



"""Invoices serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.accounts.serializers import AccountReferenceSerializer
from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    MonetaryComponentSerializer,
    MoneySerializer,
    OrganizationReferenceSerializer,
    PeriodSerializer,
)
from dfhir.chargeitems.serializers import ChargeItemReferenceSerializer
from dfhir.invoices.models import (
    Invoice,
    InvoiceLineItem,
    InvoiceParticipant,
    InvoiceParticipantActorReference,
    InvoiceRecipientReference,
    InvoiceReference,
)
from dfhir.patients.serializers import PatientGroupReferenceSerializer


class InvoiceRecipientReferenceSerializer(BaseReferenceModelSerializer):
    """Invoice recipient reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = InvoiceRecipientReference
        exclude = ["created_at", "updated_at"]


class InvoiceParticipantActorReferenceSerializer(BaseReferenceModelSerializer):
    """Invoice participant actor reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = InvoiceParticipantActorReference
        exclude = ["created_at", "updated_at"]


class InvoiceParticipantSerializer(WritableNestedModelSerializer):
    """Invoice participant serializer."""

    role = CodeableConceptSerializer(many=False, required=False)
    actor = InvoiceParticipantActorReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = InvoiceParticipant
        exclude = ["created_at", "updated_at"]


class InvoiceLineItemSerializer(WritableNestedModelSerializer):
    """Invoice line item serializer."""

    service_period = PeriodSerializer(many=False, required=False)
    charge_item_reference = ChargeItemReferenceSerializer(many=False, required=False)
    charge_item_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    price_component = MonetaryComponentSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = InvoiceLineItem
        exclude = ["created_at", "updated_at"]


class InvoiceSerializer(BaseWritableNestedModelSerializer):
    """Invoice serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    type = CodeableConceptSerializer(many=False, required=False)
    subject = PatientGroupReferenceSerializer(many=False, required=False)
    recipient = InvoiceRecipientReferenceSerializer(many=False, required=False)
    period_period = PeriodSerializer(many=False, required=False)
    participant = InvoiceParticipantSerializer(many=True, required=False)
    issuer = OrganizationReferenceSerializer(many=False, required=False)
    account = AccountReferenceSerializer(many=False, required=False)
    line_item = InvoiceLineItemSerializer(many=True, required=False)
    total_price_component = MonetaryComponentSerializer(many=True, required=False)
    total_net = MoneySerializer(many=False, required=False)
    total_gross = MoneySerializer(many=False, required=False)
    note = AnnotationSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = Invoice
        exclude = ["created_at", "updated_at"]


class InvoiceReferenceSerializer(BaseReferenceModelSerializer):
    """Invoice reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = InvoiceReference
        exclude = ["created_at", "updated_at"]



"""transport models."""

from django.db import models

from dfhir.base.models import BaseReference, Identifier, TimeStampedModel
from dfhir.transports.choices import TransportStatusChoices


class TransportReference(BaseReference):
    """transport reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_reference_identifier",
    )
    transport = models.ForeignKey(
        "Transport",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_reference_transport",
    )


class TransportBasedOnReference(BaseReference):
    """TransportBasedOn model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_based_on_identifier",
    )
    task = models.ForeignKey(
        "tasks.Task",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_based_on_task",
    )
    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_based_on_service_request",
    )
    device_request = models.ForeignKey(
        "devicerequests.DeviceRequest",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_based_on_device_request",
    )
    medication_request = models.ForeignKey(
        "medicationrequests.MedicationRequest",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_based_on_medication_request",
    )
    # TODO: requested_orchestration = models.ForeignKey(
    #     "requestedorchestrations.RequestedOrchestration",
    #     on_delete=models.DO_NOTHING,
    #     null=True,
    #     related_name="transport_based_on_requested_orchestration",
    # )
    supply_request = models.ForeignKey(
        "supplyrequests.SupplyRequest",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_based_on_supply_request",
    )
    vision_prescription = models.ForeignKey(
        "visionprescriptions.VisionPrescription",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_based_on_vision_prescription",
    )


class TransportRequesterReference(BaseReference):
    """TransportRequester model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_requester_identifier",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_requester_device",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_requester_organization",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_requester_patient",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_requester_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_requester_practitioner_role",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_requester_related_person",
    )


class TransportOwnerReference(BaseReference):
    """TransportOwnerReference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_owner_reference_identifier",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_owner_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_owner_reference_practitioner_role",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_owner_reference_organization",
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_owner_reference_care_team",
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthcareService",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_owner_reference_healthcare_service",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_owner_reference_patient",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_owner_reference_device",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_owner_reference_related_person",
    )


class TransportRestrictionRecipientReference(BaseReference):
    """transport restriction recipient model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_restriction_recipient_identifier",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_restriction_recipient_patient",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_restriction_recipient_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_restriction_recipient_practitioner_role",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_restriction_recipient_related_person",
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_restriction_recipient_group",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_restriction_recipient_organization",
    )


class TransportRestriction(TimeStampedModel):
    """transport restriction model."""

    repetitions = models.PositiveIntegerField(null=True)
    period = models.ForeignKey(
        "base.Period",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_restriction_period",
    )
    recipient = models.ManyToManyField(
        TransportRestrictionRecipientReference,
        blank=True,
        related_name="transport_restriction_recipient",
    )


class TransportInput(TimeStampedModel):
    """transport input model."""

    type = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_input_type",
    )
    value = models.CharField(max_length=255, null=True)


class TransportOutput(TimeStampedModel):
    """transport output model."""

    type = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_output_type",
    )
    value = models.CharField(max_length=255, null=True)


class Transport(TimeStampedModel):
    """transport model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="transport_identifier", blank=True
    )
    # TODO: instantiates = models.ForeignKey(
    #     "ActivityDefinitionCanonical",
    #     on_delete=models.DO_NOTHING,
    #     null=True,
    #     related_name="transport_instantiates",
    # )
    based_on = models.ManyToManyField(
        TransportBasedOnReference,
        blank=True,
        related_name="transport_based_on",
    )
    group_identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_group_identifier",
    )
    part_of = models.ManyToManyField(
        TransportReference,
        blank=True,
        related_name="transport_part_of",
    )
    status = models.CharField(
        max_length=255, null=True, choices=TransportStatusChoices.choices
    )
    status_reason = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_status_reason",
    )
    code = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_code",
    )
    description = models.TextField(null=True)
    focus = models.ForeignKey(
        "base.Reference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_focus",
    )
    for_value = models.ForeignKey(
        "base.Reference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_for_value",
    )
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_encounter",
    )
    completion_time = models.DateTimeField(null=True)
    authored_on = models.DateTimeField(null=True)
    last_modified = models.DateTimeField(null=True)
    requester = models.ForeignKey(
        TransportRequesterReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_requester",
    )
    performer_type = models.ManyToManyField(
        "base.CodeableConcept", related_name="transport_performer_type", blank=True
    )
    owner = models.ForeignKey(
        TransportOwnerReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_owner",
    )
    location = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_location",
    )
    insurance = models.ManyToManyField(
        "coverages.CoverageClaimResponseReference",
        blank=True,
        related_name="transport_insurance",
    )
    note = models.ManyToManyField(
        "base.Annotation", related_name="transport_note", blank=True
    )
    relevant_history = models.ManyToManyField(
        "provenances.ProvenanceReference",
        related_name="transport_relevant_history",
        blank=True,
    )
    restriction = models.ForeignKey(
        TransportRestriction,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_restriction",
    )
    input = models.ManyToManyField(
        TransportInput, related_name="transport_input", blank=True
    )
    output = models.ManyToManyField(
        TransportOutput, related_name="transport_output", blank=True
    )
    requested_location = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_requested_location",
    )
    current_location = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_current_location",
    )
    reason = models.ForeignKey(
        "base.CodeableReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_reason",
    )
    history = models.ForeignKey(
        TransportReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="transport_history",
    )



"""Organization serializers."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.base.serializers import (
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    ExtendedContactDetailSerializer,
    IdentifierSerializer,
    OrganizationReferenceSerializer,
    QualificationSerializer,
)
from dfhir.endpoints.serializers import EndpointReferenceSerializer
from dfhir.organizations.models import Organization, OrganizationCodeableReference


class OrganizationSerializer(BaseWritableNestedModelSerializer):
    """Organization serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    contact = ExtendedContactDetailSerializer(many=True, required=False)
    qualification = QualificationSerializer(many=True, required=False)
    type = CodeableConceptSerializer(many=True, required=False)
    endpoint = EndpointReferenceSerializer(many=True, required=False)
    part_of = OrganizationReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = Organization
        exclude = ["created_at", "updated_at"]


class OrganizationCodeableReferenceSerializer(WritableNestedModelSerializer):
    """Organization codeable reference serializer."""

    reference = OrganizationReferenceSerializer(many=False, required=False)
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta options."""

        model = OrganizationCodeableReference
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.contrib.postgres.fields
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0002_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ClaimResponseAddItem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('item_sequence', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
                ('detail_sequence', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
                ('sub_detail_sequence', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
                ('serviced_date', models.DateField(null=True)),
                ('factor', models.DecimalField(decimal_places=2, max_digits=10, null=True)),
                ('note_number', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimResponseAddItemBodySite',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimResponseAddItemDetail',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('factor', models.DecimalField(decimal_places=2, max_digits=10, null=True)),
                ('note_number', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimResponseAddItemDetailSubDetail',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('factor', models.DecimalField(decimal_places=2, max_digits=10, null=True)),
                ('note_number', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimResponseAddItemProviderReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimResponseAddItemRequestReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimResponseError',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('item_sequence', models.PositiveIntegerField(null=True)),
                ('detail_sequence', models.PositiveIntegerField(null=True)),
                ('sub_detail_sequence', models.PositiveIntegerField(null=True)),
                ('expression', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimResponseEvent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('when_date_time', models.DateTimeField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimResponseInsurance',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.PositiveIntegerField()),
                ('focal', models.BooleanField()),
                ('business_arrangement', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimResponseItem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('item_sequence', models.PositiveIntegerField()),
                ('note_number', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimResponseItemAdjudication',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimResponseItemDetail',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('detail_sequence', models.PositiveIntegerField()),
                ('note_number', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimResponseItemDetailSubDetail',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sub_detail_sequence', models.PositiveIntegerField()),
                ('note_number', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimResponseItemReviewOutcome',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('pre_auth_ref', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimResponsePayment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('date', models.DateField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimResponseProcessNote',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('number', models.PositiveIntegerField(null=True)),
                ('text', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimResponseReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimResponseRequestorReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimResponseTotal',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimResponse',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('active', 'Active'), ('cancelled', 'Cancelled'), ('draft', 'Draft'), ('entered-in-error', 'Entered in Error')], default='active', max_length=255)),
                ('use', models.CharField(choices=[('claim', 'Claim'), ('preauthorization', 'Preauthorization'), ('predetermination', 'Predetermination')], default='claim', max_length=255)),
                ('created', models.DateTimeField(null=True)),
                ('outcome', models.CharField(choices=[('queued', 'Queued'), ('complete', 'Complete'), ('error', 'Error'), ('partial', 'Partial')], max_length=255, null=True)),
                ('disposition', models.TextField(null=True)),
                ('pre_auth_ref', models.CharField(max_length=255, null=True)),
                ('decision', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_decision', to='base.codeableconcept')),
                ('diagnosis_related_group', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_diagnosis_related_group', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""molecular definitions URL Configuration."""

from django.urls import path

from dfhir.moleculardefinitions import views

app_name = "moleculardefinitions"


urlpatterns = [
    path("moleculardefinitions/", views.MolecularDefinitionListView.as_view()),
    path(
        "moleculardefinitions/<int:pk>/", views.MolecularDefinitionDetailView.as_view()
    ),
]



"""AllergyIntolerances models."""

from django.contrib.postgres.fields import ArrayField
from django.db import models

from dfhir.base.models import (
    Age,
    Annotation,
    BaseReference,
    CodeableConcept,
    Identifier,
    Period,
    Range,
    TimeStampedModel,
)

from . import choices


class AllergyIntoleranceRecorderReference(BaseReference):
    """Allergy Intolerance Recorder Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_recorder_reference_identifier",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_recorder_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_recorder_reference_practitioner_role",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_recorder_reference_patient",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_recorder_reference_related_person",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_recorder_reference_organization",
        null=True,
    )


class AllergyIntoleranceAsserterReference(BaseReference):
    """Allergy Intolerance Asserter Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_asserter_reference_identifier",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_asserter_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_asserter_reference_practitioner_role",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_asserter_reference_patient",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_asserter_reference_related_person",
        null=True,
    )


class AllergyIntoleranceReaction(TimeStampedModel):
    """Allergy Intolerance Reaction model."""

    substance = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_reaction_substance",
        null=True,
    )
    manifestation = models.ManyToManyField(
        "observations.ObservationCodeableReference",
        related_name="allergy_intolerance_reaction_manifestation",
        blank=True,
    )
    description = models.TextField(null=True)
    onset = models.DateTimeField(null=True)
    severity = models.CharField(
        max_length=255,
        choices=choices.AllergyIntoleranceSeverity.choices,
        null=True,
    )
    exposure_route = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_reaction_exposure_route",
        null=True,
    )
    note = models.ManyToManyField(
        Annotation, related_name="allergy_intolerance_reaction_note", blank=True
    )


class AllergyIntolerance(TimeStampedModel):
    """Allergy Intolerance model."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="allergy_intolerance_identifier",
        blank=True,
    )
    clinical_status = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_clinical_status",
        null=True,
    )
    verification_status = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_verification_status",
        null=True,
    )
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_type",
        null=True,
    )
    category = ArrayField(
        models.CharField(
            max_length=255,
            choices=choices.AllergyIntoleranceCategory.choices,
        ),
        null=True,
    )
    criticality = models.CharField(
        max_length=255,
        choices=choices.AllergyIntoleranceCriticality.choices,
        null=True,
    )
    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_code",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.PatientReference",
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_patient",
        null=True,
    )
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_encounter",
        null=True,
    )
    onset_date_time = models.DateTimeField(null=True)
    onset_age = models.ForeignKey(
        Age,
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_onset_age",
        null=True,
    )
    onset_period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_onset_period",
        null=True,
    )
    onset_range = models.ForeignKey(
        Range,
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_onset_range",
        null=True,
    )
    onset_string = models.CharField(max_length=255, null=True)
    recorded_date = models.DateTimeField(null=True)
    recorder = models.ForeignKey(
        AllergyIntoleranceRecorderReference,
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_recorder",
        null=True,
    )
    asserter = models.ForeignKey(
        AllergyIntoleranceAsserterReference,
        on_delete=models.CASCADE,
        related_name="allergy_intolerance_asserter",
        null=True,
    )
    last_reaction_occurrence = models.DateTimeField(null=True)
    note = models.ManyToManyField(
        Annotation, related_name="allergy_intolerance_note", blank=True
    )
    reaction = models.ManyToManyField(
        AllergyIntoleranceReaction,
        related_name="allergy_intolerance_reaction",
        blank=True,
    )


class AllergyIntoleranceReference(BaseReference):
    """allergy intolerance reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="allergy_intolerance_reference_identifier",
    )
    allergy_intolerance = models.ForeignKey(
        AllergyIntolerance,
        on_delete=models.SET_NULL,
        null=True,
        related_name="allergy_intolerance_reference_allergy_intolerance",
    )



"""Endpoint choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class EndpointStatusChoices(models.TextChoices):
    """Endpoint status choices."""

    ACTIVE = "active", _("active")
    LIMITED = "limited", _("limited")
    SUSPENDED = "suspended", _("suspended")
    ERROR = "error", _("error")
    OFF = "off", _("off")
    ENTERED_IN_ERROR = "entered-in-error", _("entered-in-error")


class EndpointEnvironmentChoices(models.TextChoices):
    """Endpoint environment choices."""

    DEVELOPMENT = "development", _("development")
    TEST = "test", _("test")
    PRODUCTION = "production", _("production")
    STAGING = "staging", _("staging")
    TRAINING = "training", _("training")



"""AppointmentResponses models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    TimeStampedModel,
)

from . import choices


class AppointmentResponseActorReference(BaseReference):
    """Appointment response actor reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="appointment_response_actor_reference_identifier",
        null=True,
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.CASCADE,
        related_name="appointment_response_actor_reference_group",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="appointment_response_actor_reference_patient",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="appointment_response_actor_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="appointment_response_actor_reference_practitioner_role",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="appointment_response_actor_reference_related_person",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.CASCADE,
        related_name="appointment_response_actor_reference_device",
        null=True,
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthCareService",
        on_delete=models.CASCADE,
        related_name="appointment_response_actor_reference_healthcare_service",
        null=True,
    )
    location = models.ForeignKey(
        "locations.Location",
        on_delete=models.CASCADE,
        related_name="appointment_response_actor_reference_location",
        null=True,
    )


class AppointmentResponse(TimeStampedModel):
    """Appointment response model."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="appointment_response_identifier",
        blank=True,
    )
    appointment = models.ForeignKey(
        "appointments.AppointmentReference",
        on_delete=models.CASCADE,
        related_name="appointment_response_appointment",
        null=True,
    )
    proposed_new_time = models.BooleanField(null=True)
    start = models.DateTimeField(null=True)
    end = models.DateTimeField(null=True)
    participant_type = models.ManyToManyField(
        CodeableConcept,
        related_name="appointment_response_participant_type",
        blank=True,
    )
    actor = models.ForeignKey(
        AppointmentResponseActorReference,
        on_delete=models.CASCADE,
        related_name="appointment_response_actor",
        null=True,
    )
    participant_status = models.CharField(
        max_length=255,
        null=True,
        choices=choices.AppointmentResponseParticipantStatusChoices.choices,
    )
    comment = models.TextField(null=True)
    recurring = models.BooleanField(null=True)
    occurrence_date = models.DateTimeField(null=True)
    recurrence_id = models.IntegerField(null=True)



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('documentreferences', '0001_initial'),
        ('medications', '0001_initial'),
        ('substances', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='MedicationKnowledgeCost',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('source', models.CharField(max_length=255, null=True)),
                ('cost_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_cost_cost_codeable_concept', to='base.codeableconcept')),
                ('cost_money', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_cost_cost_money', to='base.money')),
                ('effective_date', models.ManyToManyField(blank=True, related_name='medication_knowledge_cost_effective_date', to='base.period')),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_cost_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationKnowledgeDefinitionalDrugCharacteristic',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value_string', models.CharField(max_length=255, null=True)),
                ('value_base_64_binary', models.TextField(null=True)),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_definitional_drug_characteristic_type', to='base.codeableconcept')),
                ('value_attachment', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_definitional_drug_characteristic_value_attachment', to='base.attachment')),
                ('value_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_definitional_drug_characteristic_value_codeable_concept', to='base.codeableconcept')),
                ('value_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_definitional_drug_characteristic_value_quantity', to='base.quantity')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationKnowledgeDefinitionalIngredient',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('item', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_definitional_ingredient_item', to='substances.substancecodeablereference')),
                ('strength_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_definitional_ingredient_strength_codeable_concept', to='base.codeableconcept')),
                ('strength_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_definitional_ingredient_strength_quantity', to='base.quantity')),
                ('strength_ratio', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_definitional_ingredient_strength_ratio', to='base.ratio')),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_definitional_ingredient_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationKnowledgeDefinitional',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('dose_form', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_definitional_dose_form', to='base.codeableconcept')),
                ('intended_route', models.ManyToManyField(blank=True, related_name='medication_knowledge_definitional_intended_route', to='base.codeableconcept')),
                ('drug_characteristic', models.ManyToManyField(blank=True, related_name='medication_knowledge_definitional_drug_chracteristic', to='medicationknowledges.medicationknowledgedefinitionaldrugcharacteristic')),
                ('ingredient', models.ManyToManyField(blank=True, related_name='medication_knowledge_definitional_ingredient', to='medicationknowledges.medicationknowledgedefinitionalingredient')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationKnowledgeIndicationGuidelineDosingGuideline',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('administration_treatment', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_dosing_guideline_administration_treatment', to='base.codeableconcept')),
                ('treatment_intent', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_dosing_guideline_treatment_intent', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationKnowledgeIndicationGuideline',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('dosing_guideline', models.ManyToManyField(blank=True, related_name='medication_knowledge_indication_guidline_dosing_guideline', to='medicationknowledges.medicationknowledgeindicationguidelinedosingguideline')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationKnowledgeIndicationGuidelineDosingGuidelineDosage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_dosing_guideline_dosage_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='medicationknowledgeindicationguidelinedosingguideline',
            name='dosage',
            field=models.ManyToManyField(blank=True, related_name='medication_knowledge_dosing_guideline_dose', to='medicationknowledges.medicationknowledgeindicationguidelinedosingguidelinedosage'),
        ),
        migrations.CreateModel(
            name='MedicationKnowledgeMedicineClassification',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('source_string', models.CharField(max_length=255, null=True)),
                ('source_uri', models.URLField(null=True)),
                ('classification', models.ManyToManyField(blank=True, related_name='medication_knowledge_medicine_classification_classification', to='base.codeableconcept')),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_medicine_classification_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationKnowledgeMonitoringProgram',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=255, null=True)),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_monitoring_program_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationKnowledgeMonograph',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('source', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_monograph_source', to='documentreferences.documentreferencereference')),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_monograph_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationKnowledgePackaging',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('cost', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_packaging_cost', to='medicationknowledges.medicationknowledgecost')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationKnowledge',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('draft', 'Draft'), ('active', 'Active'), ('retired', 'Retired'), ('unknown', 'Unknown')], max_length=255, null=True)),
                ('name', models.CharField(max_length=255, null=True)),
                ('preparation_instruction', models.TextField(null=True)),
                ('associate_medication', models.ManyToManyField(blank=True, related_name='medication_knowledge_associate_medication', to='medications.medicationreference')),
                ('author', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_author', to='base.contactdetail')),
                ('code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_code', to='base.codeableconcept')),
                ('identifier', models.ManyToManyField(blank=True, related_name='medication_knowledge_identifier', to='base.identifier')),
                ('jurisdiction', models.ManyToManyField(blank=True, related_name='medication_knowledge_jurisdiction', to='base.codeableconcept')),
                ('product_type', models.ManyToManyField(blank=True, related_name='medication_knowledge_product_type', to='base.codeableconcept')),
                ('cost', models.ManyToManyField(blank=True, related_name='medication_knowledge_cost', to='medicationknowledges.medicationknowledgecost')),
                ('definitional', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_definitional', to='medicationknowledges.medicationknowledgedefinitional')),
                ('indication_guideline', models.ManyToManyField(blank=True, related_name='medication_knowledge_indication_guideline', to='medicationknowledges.medicationknowledgeindicationguideline')),
                ('medicine_classification', models.ManyToManyField(blank=True, related_name='medication_knowledge_medicine_classification', to='medicationknowledges.medicationknowledgemedicineclassification')),
                ('monitoring_program', models.ManyToManyField(blank=True, related_name='medication_knowledge_monitoring_program', to='medicationknowledges.medicationknowledgemonitoringprogram')),
                ('monograph', models.ManyToManyField(blank=True, related_name='medication_knowledge_monograph', to='medicationknowledges.medicationknowledgemonograph')),
                ('packaging', models.ManyToManyField(blank=True, related_name='medication_knowledge_packaging', to='medicationknowledges.medicationknowledgepackaging')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationKnowledgePatientCharacteristics',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='med_knowledge_dosing_guideline_treatment_intent_dosage_patient_characteristics_type', to='base.codeableconcept')),
                ('value_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='med_knowledge_dosing_guideline_treatment_intent_dosage_patient_characteristics_value_codeable_concept', to='base.codeableconcept')),
                ('value_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='med_knowledge_dosing_guideline_treatment_intent_dosage_patient_characteristics_value_quantity', to='base.quantity')),
                ('value_range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='med_knowledge_dosing_guideline_treatment_intent_dosage_patient_characteristics_value_range', to='base.range')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='medicationknowledgeindicationguidelinedosingguideline',
            name='patient_characteristics',
            field=models.ManyToManyField(blank=True, related_name='medication_knowledge_dosing_guideline_patient_characteristics', to='medicationknowledges.medicationknowledgepatientcharacteristics'),
        ),
        migrations.CreateModel(
            name='MedicationKnowledgeReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_reference_identifier', to='base.identifier')),
                ('medication_knowledge', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_reference_medication_knowledge', to='medicationknowledges.medicationknowledge')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationKnowledgeRegulatory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('regulatory_authority', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='regulatory_authority', to='base.organizationreference')),
                ('schedule', models.ManyToManyField(blank=True, related_name='medication_knowledge_schedule', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='medicationknowledge',
            name='regulatory',
            field=models.ManyToManyField(blank=True, related_name='medication_knowledge_regulatory', to='medicationknowledges.medicationknowledgeregulatory'),
        ),
        migrations.CreateModel(
            name='MedicationKnowledgeRegulatoryMaxDispense',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='regulatory_max_dispense_period', to='base.period')),
                ('quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='regulatory_max_dispense_quantity', to='base.simplequantity')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='medicationknowledgeregulatory',
            name='max_dispense',
            field=models.ManyToManyField(blank=True, related_name='regulatory_max_dispense', to='medicationknowledges.medicationknowledgeregulatorymaxdispense'),
        ),
        migrations.CreateModel(
            name='MedicationKnowledgeRegulatorySubstitution',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('allowed', models.BooleanField(null=True)),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='regulatory_substitution_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='medicationknowledgeregulatory',
            name='substitution',
            field=models.ManyToManyField(blank=True, related_name='regulatory_substitution', to='medicationknowledges.medicationknowledgeregulatorysubstitution'),
        ),
        migrations.CreateModel(
            name='MedicationKnowledgeRelatedMedicationKnowledge',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_related_medication_knowledge_reference', to='medicationknowledges.medicationknowledgereference')),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_knowledge_related_medication_knowledge_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='medicationknowledge',
            name='related_medication_knowledge',
            field=models.ManyToManyField(blank=True, related_name='medication_knowledge_related_medication_knowledge', to='medicationknowledges.medicationknowledgerelatedmedicationknowledge'),
        ),
        migrations.CreateModel(
            name='MedicationKnowledgeStorageGuideline',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.URLField(null=True)),
                ('note', models.ManyToManyField(blank=True, related_name='storage_guideline_note', to='base.annotation')),
                ('stability_duration', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='storage_guideline_stability_duration', to='base.duration')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='medicationknowledge',
            name='storage_guideline',
            field=models.ManyToManyField(blank=True, related_name='medication_knowledge_storage_guideline', to='medicationknowledges.medicationknowledgestorageguideline'),
        ),
        migrations.CreateModel(
            name='MedicationKnowledgeStorageGuidelineEnvironmentalSetting',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='base.codeableconcept')),
                ('value_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='storage_guideline_environmental_setting_value_codeable_concept', to='base.codeableconcept')),
                ('value_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='storage_guideline_environmental_setting_value_quantity', to='base.quantity')),
                ('value_range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='storage_guideline_environmental_setting_value_range', to='base.range')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='medicationknowledgestorageguideline',
            name='environmental_setting',
            field=models.ManyToManyField(blank=True, related_name='storage_guideline_environmental_setting', to='medicationknowledges.medicationknowledgestorageguidelineenvironmentalsetting'),
        ),
    ]



"""observation definitions serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    CodingSerializer,
    ContactDetailSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    RangeSerializer,
    UsageContextSerializer,
)
from dfhir.observationdefinitions.models import (
    ObservationDefinition,
    ObservationDefinitionComponent,
    ObservationDefinitionQualifiedValue,
    ObservationDefinitionQuestionnaireReference,
    ObservationDefinitionReference,
)
from dfhir.specimendefinitions.serializers import SpecimenDefinitionReferenceSerializer


class ObservationDefinitionQualifiedValueSerializer(WritableNestedModelSerializer):
    """Serializer for observation definition qualified values."""

    context = CodeableConceptSerializer(many=False, required=False)
    applies_to = CodeableConceptSerializer(many=False, required=False)
    age = RangeSerializer(many=False, required=False)
    gestational_age = RangeSerializer(many=False, required=False)
    range = RangeSerializer(many=False, required=False)
    interpretation = CodeableConceptSerializer(many=True, required=False)

    class Meta:
        """meta options."""

        model = ObservationDefinitionQualifiedValue
        exclude = ["created_at", "updated_at"]


class ObservationDefinitionComponentSerializer(WritableNestedModelSerializer):
    """Serializer for observation definition components."""

    code = CodeableConceptSerializer(many=False, required=False)
    permitted_unit = CodingSerializer(many=True, required=False)
    qualified_value = ObservationDefinitionQualifiedValueSerializer(
        many=True, required=False
    )

    class Meta:
        """meta options."""

        model = ObservationDefinitionComponent
        exclude = ["created_at", "updated_at"]


class ObservationDefinitionQuestionnaireReferenceSerializer(
    BaseReferenceModelSerializer
):
    """Serializer for observation definition questionnaire references."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = ObservationDefinitionQuestionnaireReference
        exclude = ["created_at", "updated_at"]


class ObservationDefinitionSerializer(BaseWritableNestedModelSerializer):
    """Serializer for observation definitions."""

    identifier = IdentifierSerializer(many=False, required=False)
    version_algorithm_coding = CodingSerializer(many=False, required=False)
    contact = ContactDetailSerializer(many=True, required=False)
    use_context = UsageContextSerializer(many=True, required=False)
    jurisdiction = CodingSerializer(many=True, required=False)
    effective_period = PeriodSerializer(many=False, required=False)
    subject = CodingSerializer(many=True, required=False)
    performer_type = CodeableConceptSerializer(many=False, required=False)
    category = CodeableConceptSerializer(many=True, required=False)
    code = CodeableConceptSerializer(many=False, required=False)
    body_site = CodeableConceptSerializer(many=True, required=False)
    method = CodeableConceptSerializer(many=False, required=False)
    specimen = SpecimenDefinitionReferenceSerializer(required=False)
    permitted_unit = CodingSerializer(many=True, required=False)
    qualified_value = ObservationDefinitionQualifiedValueSerializer(
        many=True, required=False
    )
    has_member = ObservationDefinitionQuestionnaireReferenceSerializer(
        many=True, required=False
    )
    component = ObservationDefinitionComponentSerializer(many=True, required=False)

    class Meta:
        """meta options."""

        model = ObservationDefinition
        exclude = ["created_at", "updated_at"]


class ObservationDefinitionReferenceSerializer(BaseReferenceModelSerializer):
    """Serializer for observation definition references."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = ObservationDefinitionReference
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('conditions', '0004_initial'),
        ('coverageeligibilityrequests', '0001_initial'),
        ('coverages', '0002_initial'),
        ('locations', '0001_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0001_initial'),
        ('practitionerroles', '0001_initial'),
        ('practitioners', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='coverageeligibilityrequest',
            name='enterer',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_enterer', to='practitioners.practitionerpractitionerrolereference'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequest',
            name='facility',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_facility', to='locations.locationreference'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequest',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='coverage_eligibility_request_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequest',
            name='insurer',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_insurer', to='base.organizationreference'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequest',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_patient', to='patients.patientreference'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequest',
            name='priority',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_priority', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequest',
            name='service_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_service_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestevent',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_event_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestevent',
            name='when_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_event_when_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequest',
            name='event',
            field=models.ManyToManyField(blank=True, related_name='coverage_eligibility_request_event', to='coverageeligibilityrequests.coverageeligibilityrequestevent'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestinsurance',
            name='coverage',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_insurance_coverage', to='coverages.coveragereference'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequest',
            name='insurance',
            field=models.ManyToManyField(blank=True, related_name='coverage_eligibility_request_insurance', to='coverageeligibilityrequests.coverageeligibilityrequestinsurance'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestitem',
            name='category',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_item_category', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestitem',
            name='facility',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_item_facility', to='locations.locationorganizationreference'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestitem',
            name='modifier',
            field=models.ManyToManyField(blank=True, related_name='coverage_eligibility_request_item_modifier', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestitem',
            name='product_or_service',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_item_product_or_service', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestitem',
            name='provider',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_item_provider', to='practitioners.practitionerpractitionerrolereference'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestitem',
            name='quantity',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_item_quantity', to='base.simplequantity'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestitem',
            name='unit_price',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_item_unit_price', to='base.money'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequest',
            name='item',
            field=models.ManyToManyField(blank=True, related_name='coverage_eligibility_request_item', to='coverageeligibilityrequests.coverageeligibilityrequestitem'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestitemdetailreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_item_detail_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestitem',
            name='detail',
            field=models.ManyToManyField(blank=True, related_name='coverage_eligibility_request_item_detail', to='coverageeligibilityrequests.coverageeligibilityrequestitemdetailreference'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestitemdiagnosis',
            name='diagnosis_codeable_concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_item_diagnosis_codeable_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestitemdiagnosis',
            name='diagnosis_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_item_diagnosis_reference', to='conditions.conditionreference'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestitem',
            name='diagnosis',
            field=models.ManyToManyField(blank=True, related_name='coverage_eligibility_request_item_diagnosis', to='coverageeligibilityrequests.coverageeligibilityrequestitemdiagnosis'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestproviderreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_provider_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestproviderreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_provider_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestproviderreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_provider_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestproviderreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_provider_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequest',
            name='provider',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_provider', to='coverageeligibilityrequests.coverageeligibilityrequestproviderreference'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequest',
            name='supporting_info',
            field=models.ManyToManyField(blank=True, related_name='coverage_eligibility_request_supporting_info', to='coverageeligibilityrequests.coverageeligibilityrequestsupportinginfo'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestsupportinginfoinformationreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_supporting_info_information_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityrequestsupportinginfo',
            name='information',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_request_supporting_info_information', to='coverageeligibilityrequests.coverageeligibilityrequestsupportinginfoinformationreference'),
        ),
    ]



"""risk assessment model choices."""

from django.db.models import TextChoices
from django.utils.translation import gettext_lazy as _


class RiskAssessmentStatusChoices(TextChoices):
    """risk assessment status choices."""

    REGISTERED = "registered", _("Registered")
    SPECIMEN_IN_PROGRESS = "specimen-in-progress", _("Specimen in Progress")
    PRELIMINARY = "preliminary", _("Preliminary")
    FINAL = "final", _("Final")
    AMENDED = "amended", _("Amended")
    CORRECTED = "corrected", _("Corrected")
    APPENDED = "appended", _("Appended")
    CANCELLED = "cancelled", _("Cancelled")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")
    UNKNOWN = "unknown", _("Unknown")
    CANNOT_BE_OBTAINED = "cannot-be-obtained", _("Con not be Obtained")



"""molecular sequences models."""

from django.db import models

from dfhir.base.models import (
    Attachment,
    BaseReference,
    CodeableConcept,
    Identifier,
    TimeStampedModel,
)
from dfhir.molecularsequences.choices import MolecularSequenceTypeChoices


class MolecularSequenceReference(BaseReference):
    """Molecular sequence reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        related_name="molecular_sequence_reference_identifier",
        null=True,
    )
    molecular_sequence = models.ForeignKey(
        "MolecularSequence",
        on_delete=models.SET_NULL,
        related_name="molecular_sequence_reference_molecular_sequence",
        null=True,
    )


class MolecularSequenceLiteral(TimeStampedModel):
    """Molecular sequence literal model."""

    sequence_value = models.CharField(max_length=255, null=True)


class MolecularSequenceRelativeEdit(TimeStampedModel):
    """Molecular sequence relative edit model."""

    edit_order = models.IntegerField()
    coordinate_system = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="molecular_sequence_relative_edit_coordinate_system",
    )
    start = models.IntegerField(null=True)
    end = models.IntegerField(null=True)
    replacement_sequence = models.ForeignKey(
        MolecularSequenceReference,
        on_delete=models.DO_NOTHING,
        related_name="molecular_sequence_relative_edit_replacement_sequence",
    )
    replaced_sequence = models.ForeignKey(
        MolecularSequenceReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="molecular_sequence_relative_edit_replaced_sequence",
    )


class MolecularSequenceRelative(TimeStampedModel):
    """Molecular sequence relative model."""

    starting_sequence = models.ForeignKey(
        MolecularSequenceReference,
        on_delete=models.SET_NULL,
        related_name="molecular_sequence_relative_starting_sequence",
        null=True,
    )
    edit = models.ManyToManyField(
        MolecularSequenceRelativeEdit,
        blank=True,
        related_name="molecular_sequence_relative_edit",
    )


class MolecularSequenceExtracted(TimeStampedModel):
    """Molecular sequence extracted model."""

    starting_sequence = models.ForeignKey(
        MolecularSequenceReference,
        on_delete=models.SET_NULL,
        related_name="molecular_sequence_extracted_starting_sequence",
        null=True,
    )
    start = models.IntegerField(null=True)
    end = models.IntegerField(null=True)
    coordinated_system = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="molecular_sequence_extracted_coordinated_system",
    )
    reverse_complement = models.BooleanField(default=False)


class MolecularSequenceRepeated(TimeStampedModel):
    """Molecular sequence repeated model."""

    sequence_motif = models.ForeignKey(
        MolecularSequenceReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="molecular_sequence_repeated_sequence_motif",
    )
    copy_count = models.IntegerField(null=True)


class MolecularSequenceConcatenatedSequenceElement(TimeStampedModel):
    """Molecular sequence concatenated sequence element model."""

    sequence = models.ForeignKey(
        MolecularSequenceReference,
        null=True,
        related_name="molecular_sequence_concatenated_sequence",
        on_delete=models.SET_NULL,
    )
    ordinate_index = models.IntegerField(null=True)


class MolecularSequenceConcatenated(TimeStampedModel):
    """Molecular sequence concatenated model."""

    sequence_element = models.ManyToManyField(
        MolecularSequenceConcatenatedSequenceElement,
        blank=True,
        related_name="molecular_sequence_concatenated_sequence_element",
    )


class MolecularSequence(TimeStampedModel):
    """Molecular sequence model."""

    identifier = models.ManyToManyField(
        Identifier, blank=True, related_name="molecular_sequence_identifier"
    )
    type = models.CharField(
        max_length=255, null=True, choices=MolecularSequenceTypeChoices.choices
    )
    literal = models.ManyToManyField(
        MolecularSequenceLiteral, blank=True, related_name="molecular_sequence_literal"
    )
    file = models.ManyToManyField(
        Attachment, blank=True, related_name="molecular_sequence_file"
    )
    relative = models.ManyToManyField(
        MolecularSequenceRelative,
        blank=True,
        related_name="molecular_sequence_relative",
    )
    extracted = models.ManyToManyField(
        MolecularSequenceExtracted,
        blank=True,
        related_name="molecular_sequence_extracted",
    )
    repeated = models.ManyToManyField(
        MolecularSequenceRepeated,
        blank=True,
        related_name="molecular_sequence_repeated",
    )
    concatenated = models.ManyToManyField(
        MolecularSequenceConcatenated,
        blank=True,
        related_name="molecular_sequence_concatenated",
    )



"""Appointment responses urls."""

from django.urls import path

from . import views

app_name = "appointmentresponses"

urlpatterns = [
    path(
        "appointmentresponses/",
        views.AppointmentResponseListView.as_view(),
        name="list-create",
    ),
    path(
        "appointmentresponses/<int:pk>/",
        views.AppointmentResponseDetailView.as_view(),
        name="detail",
    ),
]



"""immunization recommendation urls."""

from django.urls import path

from dfhir.immunizationrecommendations.views import (
    ImmunizationRecommendationDetailView,
    ImmunizationRecommendationListView,
)

app_name = "immunizationrecommendations"

urlpatterns = [
    path(
        "immunizationrecommendations/",
        ImmunizationRecommendationListView.as_view(),
        name="immunizationrecommendation-list",
    ),
    path(
        "immunizationrecommendations/<int:pk>/",
        ImmunizationRecommendationDetailView.as_view(),
        name="immunizationrecommendation-detail",
    ),
]



"""flags models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    Period,
    TimeStampedModel,
)

from . import choices


class FlagSubjectReference(BaseReference):
    """Flag Subject Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="flag_subject_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="flag_subject_reference_patient",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="flag_subject_reference_related_person",
        null=True,
    )
    location = models.ForeignKey(
        "locations.Location",
        on_delete=models.CASCADE,
        related_name="flag_subject_reference_location",
        null=True,
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.CASCADE,
        related_name="flag_subject_reference_group",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="flag_subject_reference_organization",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="flag_subject_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="flag_subject_reference_practitioner_role",
        null=True,
    )
    # plan_definition = models.ForeignKey("plandefinitions.PlanDefinition", on_delete=models.CASCADE, related_name="flag_subject_reference_plan_definition", null=True)
    medication = models.ForeignKey(
        "medications.Medication",
        on_delete=models.CASCADE,
        related_name="flag_subject_reference_medication",
        null=True,
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.CASCADE,
        related_name="flag_subject_reference_procedure",
        null=True,
    )


class FlagAuthorReference(BaseReference):
    """Flag Author Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="flag_author_reference_identifier",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.CASCADE,
        related_name="flag_author_reference_device",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="flag_author_reference_organization",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="flag_author_reference_patient",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="flag_author_reference_related_person",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="flag_author_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="flag_author_reference_practitioner_role",
        null=True,
    )


class FlagSupportingInfoReference(BaseReference):
    """Flag Supporting Info Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="flag_supporting_info_reference_identifier",
        null=True,
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.CASCADE,
        related_name="flag_supporting_info_reference_observation",
        null=True,
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        on_delete=models.CASCADE,
        related_name="flag_supporting_info_reference_condition",
        null=True,
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.CASCADE,
        related_name="flag_supporting_info_reference_procedure",
        null=True,
    )
    allergy_intolerance = models.ForeignKey(
        "allergyintolerances.AllergyIntolerance",
        on_delete=models.CASCADE,
        related_name="flag_supporting_info_reference_allergy_intolerance",
        null=True,
    )
    risk_assessment = models.ForeignKey(
        "riskassessments.RiskAssessment",
        on_delete=models.CASCADE,
        related_name="flag_supporting_info_reference_risk_assessment",
        null=True,
    )


class Flag(TimeStampedModel):
    """Flag model."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="flag_identifier",
        blank=True,
    )
    status = models.CharField(
        max_length=255, choices=choices.FlagStatusChoices.choices, null=True
    )
    category = models.ManyToManyField(
        CodeableConcept, related_name="flag_category", blank=True
    )
    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="flag_code",
        null=True,
    )
    subject = models.ForeignKey(
        FlagSubjectReference,
        on_delete=models.CASCADE,
        related_name="flag_subject",
        null=True,
    )
    period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="flag_period",
        null=True,
    )
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.CASCADE,
        related_name="flag_encounter",
        null=True,
    )
    author = models.ForeignKey(
        FlagAuthorReference,
        on_delete=models.CASCADE,
        related_name="flag_author",
        null=True,
    )
    supporting_info = models.ManyToManyField(
        FlagSupportingInfoReference,
        related_name="flag_supports",
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.contrib.postgres.fields
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0002_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ClaimAccident',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('date', models.DateField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimCareTeam',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.IntegerField(null=True)),
                ('responsible', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimCareTeamProviderReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimDiagnosis',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.IntegerField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimEntererReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimEvent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('when_date_time', models.DateTimeField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimInsurance',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.IntegerField(null=True)),
                ('focal', models.BooleanField(default=False)),
                ('business_arrangement', models.CharField(max_length=255, null=True)),
                ('pre_auth_ref', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimItem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.IntegerField(null=True)),
                ('care_team_sequence', django.contrib.postgres.fields.ArrayField(base_field=models.IntegerField(), null=True, size=None)),
                ('diagnosis_sequence', django.contrib.postgres.fields.ArrayField(base_field=models.IntegerField(), null=True, size=None)),
                ('procedure_sequence', django.contrib.postgres.fields.ArrayField(base_field=models.IntegerField(), null=True, size=None)),
                ('information_sequence', django.contrib.postgres.fields.ArrayField(base_field=models.IntegerField(), null=True, size=None)),
                ('serviced_date', models.DateField(null=True)),
                ('factor', models.FloatField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimItemBodySIte',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimItemDetail',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.IntegerField(null=True)),
                ('factor', models.FloatField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimItemDetailSubDetail',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.IntegerField(null=True)),
                ('factor', models.FloatField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimItemRequestReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimOriginalPrescriptionReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimPayee',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimPayeePartyReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimPrescriptionReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimProcedure',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.IntegerField(null=True)),
                ('date', models.DateField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimProviderReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimRelated',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClaimSupportingInfo',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.IntegerField(null=True)),
                ('timing_date', models.DateField(null=True)),
                ('value_boolean', models.BooleanField(null=True)),
                ('value_string', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Claim',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_created=True, null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('active', 'Active'), ('cancelled', 'Cancelled'), ('draft', 'Draft'), ('entered-in-error', 'Entered in Error')], default='draft', max_length=255)),
                ('use', models.CharField(choices=[('claim', 'Claim'), ('preauthorization', 'Preauthorization'), ('predetermination', 'Predetermination')], default='claim', max_length=255)),
                ('billable_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_billable_period', to='base.period')),
                ('diagnosis_related_group', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_diagnosis_related_group', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""medication url configurations."""

from django.urls import path

from dfhir.medications.views import (
    MedicationCodesList,
    MedicationDetail,
    MedicationDoseFormList,
    MedicationIngredientItemList,
    MedicationsList,
)

app_name = "medications"
urlpatterns = [
    path("medications/", MedicationsList.as_view(), name="medications-list"),
    path(
        "medications/<int:pk>/", MedicationDetail.as_view(), name="medications-detail"
    ),
    path(
        "medication-ingredient-items/",
        MedicationIngredientItemList.as_view(),
        name="medication-ingredient-items-list",
    ),
    path(
        "medication-dose-forms/",
        MedicationDoseFormList.as_view(),
        name="medication-dose-forms-list",
    ),
    path(
        "medication-codes/", MedicationCodesList.as_view(), name="medication-codes-list"
    ),
]
